﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Posloupnost
{
    class AnalyzaPosloupnosti
    {
        public AnalyzaPosloupnosti()
        {

        }

        /// <summary>
        /// zde probíhá hlavní výpočet dalších čelnů posloupnosti
        /// </summary>
        /// <param name="posloupnost"></param>
        /// <returns></returns>
        public Posloupnost AnalyzujPosloupnost(Posloupnost posloupnost)
        {
            posloupnost.Diference = SpocitejDiference(posloupnost.VstupniCleny);

            //aritmetická a geometrická posloupnost
            posloupnost.DalsiCleny = AritmetickaPosloupnost(posloupnost);
            if (!posloupnost.DalsiCleny.Any())
                posloupnost.DalsiCleny = GeometrickaPosloupnost(posloupnost);

            //zde se počítá posloupnost, která je složená ze dvou posloupností
            if (!posloupnost.DalsiCleny.Any())
            {
                Posloupnost licheCleny = new Posloupnost();
                Posloupnost sudeCleny = new Posloupnost();

                for (int i = 0; i < posloupnost.VstupniCleny.Count; i++)
                {
                    if (i % 2 == 0)
                        sudeCleny.VstupniCleny.Add(posloupnost.VstupniCleny[i]);
                    else
                        licheCleny.VstupniCleny.Add(posloupnost.VstupniCleny[i]);
                }
                licheCleny.Diference = SpocitejDiference(licheCleny.VstupniCleny);
                sudeCleny.Diference = SpocitejDiference(sudeCleny.VstupniCleny);

                licheCleny.DalsiCleny = AritmetickaPosloupnost(licheCleny);
                if (!licheCleny.DalsiCleny.Any())
                    licheCleny.DalsiCleny = GeometrickaPosloupnost(licheCleny);

                sudeCleny.DalsiCleny = AritmetickaPosloupnost(sudeCleny);
                if (!sudeCleny.DalsiCleny.Any())
                    sudeCleny.DalsiCleny = GeometrickaPosloupnost(sudeCleny);

                posloupnost.DalsiCleny = ZkombinujPosloupnosti(sudeCleny.DalsiCleny, licheCleny.DalsiCleny);
            }

            //zde bych ještě zkusil dodělat Fibonacciho posloupnost, prvočísla a varianty těchto posloupností, ale už nemám čas

            return posloupnost;
        }

        /// <summary>
        /// testuje (a případně dopočítává), zda je posloupnost aritmetická (případně diference posloupnosti)
        /// </summary>
        /// <param name="posloupnost"></param>
        /// <returns></returns>
        public List<int> AritmetickaPosloupnost(Posloupnost posloupnost)
        {
            if (JeAritmetickaPosloupnost(posloupnost.VstupniCleny))
            {
                posloupnost.DalsiCleny = DopocitejAritmetickouPosloupnost(posloupnost.VstupniCleny, posloupnost.Diference);
            }
            //zjištění, zda diference tvoří aritmetickou posloupnost + případné dopočítání dalších diferencí a členů původní posloupnosti
            else if (JeAritmetickaPosloupnost(posloupnost.Diference))
            {
                List<int> diferenceDiferenci = SpocitejDiference(posloupnost.Diference);
                List<int> dalsiDiference = DopocitejAritmetickouPosloupnost(posloupnost.Diference, diferenceDiferenci);
                posloupnost.DalsiCleny = DopocitejAritmetickouPosloupnost(posloupnost.VstupniCleny, dalsiDiference);
            }
            return posloupnost.DalsiCleny;
        }

        /// <summary>
        /// testuje (a případně dopočítává), zda je posloupnost geometrická (případně diference posloupnosti)
        /// </summary>
        /// <param name="posloupnost"></param>
        /// <returns></returns>
        public List<int> GeometrickaPosloupnost(Posloupnost posloupnost)
        {
            if (JeGeometrickaPosloupnost(posloupnost.VstupniCleny))
            {
                posloupnost.DalsiCleny = DopocitejGeometrickouPosloupnost(posloupnost.VstupniCleny);
            }
            //zjištění, zda diference tvoří geometrickou posloupnost + případné dopočítání dalších diferencí a členů původní posloupnosti
            else if (JeGeometrickaPosloupnost(posloupnost.Diference))
            {
                List<int> dalsiDiference = DopocitejGeometrickouPosloupnost(posloupnost.Diference);
                posloupnost.DalsiCleny = DopocitejAritmetickouPosloupnost(posloupnost.VstupniCleny, dalsiDiference);
            }

            return posloupnost.DalsiCleny;
        }
    
        public bool JeAritmetickaPosloupnost(List<int> posloupnost)
        {
            bool jeAritmeticka = true;
            List<int> diference = SpocitejDiference(posloupnost);

            //-2 je zde z toho důvodu, že jedna diference je "navíc"
            for (int i = 0; i < diference.Count - 2; i++)
            {
                if (diference[0] != diference[i])
                    jeAritmeticka = false;
            }

            return jeAritmeticka;
        }

        /// <summary>
        /// dopočítává aritmetickou posloupnost
        /// </summary>
        /// <param name="vstupniCleny"></param>
        /// <param name="diference"></param>
        /// <returns></returns>
        public List<int> DopocitejAritmetickouPosloupnost(List<int> vstupniCleny, List<int> diference)
        {
            int pocetClenu = vstupniCleny.Count;
            List<int> dalsiCleny = new List<int>();

            dalsiCleny.Add(vstupniCleny[pocetClenu - 1] + diference[0]);

            for (int i = 0; i < vstupniCleny.Count - 1; i++)
            {
                dalsiCleny.Add(dalsiCleny[i] + diference[i + 1]);
            }

            return dalsiCleny;
        }


        public bool JeGeometrickaPosloupnost(List<int> posloupnost)
        {
            bool jeGeometricka = true;
            List<double> kvocienty = new List<double>();

            if (!posloupnost.Contains(0))
            {
                //-2 je zde z toho důvodu, že jedna diference je "navíc"
                for (int i = 0; i < posloupnost.Count - 2; i++)
                {
                    kvocienty.Add((double)posloupnost[i + 1] / (double)posloupnost[i]);
                }
            }
            else
                jeGeometricka = false;
            
            foreach (double kvocient in kvocienty)
            {
                if (kvocient != kvocienty[0])
                    jeGeometricka = false;
            }

            return jeGeometricka;
        }

        /// <summary>
        /// dopočítává geometrickou posloupnost
        /// </summary>
        /// <param name="vstupniCleny"></param>
        /// <returns></returns>
        public List<int> DopocitejGeometrickouPosloupnost(List<int> vstupniCleny)
        {
            int pocetClenu = vstupniCleny.Count;
            List<int> dalsiCleny = new List<int>();
            double kvocient = vstupniCleny[1] / vstupniCleny[0]; 

            dalsiCleny.Add(Convert.ToInt32(vstupniCleny[pocetClenu - 1] * kvocient));

            for (int i = 0; i < vstupniCleny.Count - 1; i++)
            {
                dalsiCleny.Add(Convert.ToInt32(dalsiCleny[i] * kvocient));
            }

            return dalsiCleny;
        }

        /// <summary>
        /// vypočítá diference posloupnosti
        /// </summary>
        /// <param name="cleny"></param>
        /// <returns></returns>
        private List<int> SpocitejDiference(List<int> cleny)
        {
            List<int> diference = new List<int>();

            for (int i = 0; i < cleny.Count - 1; i++)
            {
                diference.Add(cleny[i + 1] - cleny[i]);
            }

            //vzhledem k tomu, že počítání diference je "ztrátové" (z 8 členů posloupnosti získám 7 diferencí) tak zde přidávám jednu navíc (abych dorovnal ztrátu)
            diference.Add(cleny[cleny.Count - 1] - cleny[cleny.Count - 2]);

            return diference;
        }

        /// <summary>
        /// kombinuje 2 posloupnosti do jedné
        /// </summary>
        /// <param name="sudeCleny"></param>
        /// <param name="licheCleny"></param>
        /// <returns></returns>
        private List<int> ZkombinujPosloupnosti(List<int> sudeCleny, List<int> licheCleny)
        {
            List<int> dalsiCleny = new List<int>();

            for (int i = 0; i < licheCleny.Count; i++)
            {
                dalsiCleny.Add(sudeCleny[i]);
                dalsiCleny.Add(licheCleny[i]);
            }
            if (sudeCleny.Count > licheCleny.Count)
            {
                dalsiCleny.Add(sudeCleny[sudeCleny.Count - 1]);
            }

            return dalsiCleny;
        }
    }
}
